from .provider import AbstractProvider  # noqa: F401 noreorder
from .alibaba_provider import AlibabaProvider  # noqa: F401 noreorder
from .aws_provider import AWSProvider  # noqa: F401 noreorder
from .gcp_provider import GCPProvider  # noqa: F401 noreorder
from .do_provider import DOProvider  # noqa: F401 noreorder
from .azure_provider import AzureProvider  # noqa: F401 noreorder
from .oci_provider import OCIProvider  # noqa: F401 noreorder



import logging
from pathlib import Path

import requests

from . import AbstractProvider


class GCPProvider(AbstractProvider):
    """
        Concrete implementation of the GCP cloud provider.
    """

    def __init__(self, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.metadata_url = (
            'http://metadata.google.internal/computeMetadata/v1/instance/tags'
        )
        self.vendor_file = '/sys/class/dmi/id/product_name'
        self.headers = {'Metadata-Flavor': 'Google'}

    def identify(self):
        """
            Tries to identify GCP using all the implemented options
        """
        self.logger.info('Try to identify GCP')
        return self.check_metadata_server() or self.check_vendor_file()

    def check_metadata_server(self):
        """
            Tries to identify GCP via metadata server
        """
        self.logger.debug('Checking GCP metadata')
        try:
            requests.get(
                self.metadata_url, headers=self.headers,
            )
            return True
        except requests.exceptions.RequestException as e:  # noqa: F841
            return False

    def check_vendor_file(self):
        """
            Tries to identify GCP provider by reading the /sys/class/dmi/id/product_name
        """
        self.logger.debug('Checking GCP vendor file')
        gcp_path = Path(self.vendor_file)
        if gcp_path.is_file():
            if 'Google' in open(self.vendor_file).read():
                return True
        return False



import logging

from cloud_detect.providers import AlibabaProvider
from cloud_detect.providers import AWSProvider
from cloud_detect.providers import AzureProvider
from cloud_detect.providers import DOProvider
from cloud_detect.providers import GCPProvider
from cloud_detect.providers import OCIProvider


def provider(excluded=[]):
    if 'alibaba' not in excluded and AlibabaProvider().identify():
        logging.debug('Cloud_detect result is alibaba')
        return 'alibaba'
    elif 'aws' not in excluded and AWSProvider().identify():
        logging.debug('Cloud_detect result is aws')
        return 'aws'
    elif 'gcp' not in excluded and GCPProvider().identify():
        logging.debug('Cloud_detect result is gcp')
        return 'gcp'
    elif 'do' not in excluded and DOProvider().identify():
        logging.debug('Cloud_detect result is do')
        return 'do'
    elif 'azure' not in excluded and AzureProvider().identify():
        logging.debug('Cloud_detect result is azure')
        return 'azure'
    elif 'oci' not in excluded and OCIProvider().identify():
        logging.debug('Cloud_detect result is oci')
        return 'oci'
    else:
        logging.debug('Cloud_detect result is unknown')
        return 'unknown'



import logging
from pathlib import Path

import requests

from . import AbstractProvider


class AWSProvider(AbstractProvider):
    """
        Concrete implementation of the AWS cloud provider.
    """

    def __init__(self, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.metadata_url = (
            'http://169.254.169.254/latest/dynamic/instance-identity/document'
        )
        self.vendor_file = '/sys/class/dmi/id/product_version'

    def identify(self):
        """
            Tries to identify AWS using all the implemented options
        """
        self.logger.info('Try to identify AWS')
        return self.check_metadata_server() or self.check_vendor_file()

    def check_metadata_server(self):
        """
            Tries to identify AWS via metadata server
        """
        self.logger.debug('Checking AWS metadata')
        try:
            response = requests.get(self.metadata_url).json()
            if response['imageID'].startswith('ami-',) and response[
                'instanceId'
            ].startswith('i-'):
                return True
            return False
        except BaseException:
            return False

    def check_vendor_file(self):
        """
            Tries to identify AWS provider by reading the /sys/class/dmi/id/product_version
        """
        self.logger.debug('Checking AWS vendor file')
        aws_path = Path(self.vendor_file)
        if aws_path.is_file():
            if 'amazon' in open(self.vendor_file).read():
                return True
        return False



import logging
from pathlib import Path

import requests

from . import AbstractProvider


class AlibabaProvider(AbstractProvider):
    """
        Concrete implementation of the AWS cloud provider.
    """

    def __init__(self, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.metadata_url = 'http://100.100.100.200/latest/meta-data/latest/meta-data/instance/virtualization-solution'  # noqa
        self.vendor_file = '/sys/class/dmi/id/product_name'

    def identify(self):
        """
            Tries to identify Alibaba using all the implemented options
        """
        self.logger.info('Try to identify Alibaba')
        return self.check_metadata_server() or self.check_vendor_file()

    def check_metadata_server(self):
        """
            Tries to identify Alibaba via metadata server
        """
        self.logger.debug('Checking Alibaba metadata')
        try:
            response = requests.get(self.metadata_url)
            if response.text == 'ECS Virt':
                return True
            return False
        except BaseException:
            return False

    def check_vendor_file(self):
        """
            Tries to identify Alibaba provider by reading the /sys/class/dmi/id/product_name
        """
        self.logger.debug('Checking Aliobaba vendor file')
        alibaba_path = Path(self.vendor_file)
        if alibaba_path.is_file():
            if 'Alibaba Cloud ECS' in open(self.vendor_file).read():
                return True
        return False



import logging
from pathlib import Path

import requests

from . import AbstractProvider


class DOProvider(AbstractProvider):
    """
        Concrete implementation of the Digital Ocean cloud provider.
    """

    def __init__(self, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.metadata_url = 'http://169.254.169.254/metadata/v1.json'
        self.vendor_file = '/sys/class/dmi/id/sys_vendor'

    def identify(self):
        """
            Tries to identify DO using all the implemented options
        """
        self.logger.info('Try to identify DO')
        return self.check_metadata_server() or self.check_vendor_file()

    def check_metadata_server(self):
        """
            Tries to identify DO via metadata server
        """
        self.logger.debug('Checking DO metadata')
        try:
            response = requests.get(self.metadata_url).json()
            if response['droplet_id'] > 0:
                return True
            return False
        except BaseException:
            return False

    def check_vendor_file(self):
        """
            Tries to identify DO provider by reading the /sys/class/dmi/id/sys_vendor
        """
        self.logger.debug('Checking DO vendor file')
        do_path = Path(self.vendor_file)
        if do_path.is_file():
            if 'DigitalOcean' in open(self.vendor_file).read():
                return True
        return False



import logging
from pathlib import Path

from . import AbstractProvider


class OCIProvider(AbstractProvider):
    """
        Concrete implementation of the Oracle Cloud Infrastructure cloud provider.
    """

    def __init__(self, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.vendor_file = '/sys/class/dmi/id/chassis_asset_tag'

    def identify(self):
        """
            Tries to identify OCI using all the implemented options
        """
        self.logger.info('Try to identify OCI')
        return self.check_vendor_file()

    def check_metadata_server(self):
        raise NotImplementedError

    def check_vendor_file(self):
        """
            Tries to identify OCI provider by reading the file -> /sys/class/dmi/id/chassis_asset_tag # noqa
        """
        self.logger.debug('Checking OCI vendor file')
        oci_path = Path(self.vendor_file)
        if oci_path.is_file():
            if 'OracleCloud' in open(self.vendor_file).read():
                return True
        return False



from distutils.core import setup

import setuptools

long_description = ''
try:
    with open('README.md') as f:
        long_description = f.read()
except FileNotFoundError:
    pass

setup(
    name='cloud-detect',
    version='0.0.6',
    description="Module that determines a host's cloud provider",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/dgzlopes/cloud-detect',
    license='MIT',
    install_requires=[
        'requests>=2.21.0,<3',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Systems Administration',
        'Topic :: System :: Networking',
    ],
    python_requires='>=3.4',
    author='Daniel Gonzalez Lopes',
    author_email='danielgonzalezlopes@gmail.com',
    packages=setuptools.find_packages(),
)



import logging
from pathlib import Path

import requests

from . import AbstractProvider


class AzureProvider(AbstractProvider):
    """
        Concrete implementation of the Azure cloud provider.
    """

    def __init__(self, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.metadata_url = (
            'http://169.254.169.254/metadata/instance?api-version=2017-12-01'
        )
        self.vendor_file = '/sys/class/dmi/id/sys_vendor'
        self.headers = {'Metadata': 'true'}

    def identify(self):
        """
            Tries to identify Azure using all the implemented options
        """
        self.logger.info('Try to identify DO')
        return self.check_metadata_server() or self.check_vendor_file()

    def check_metadata_server(self):
        """
            Tries to identify Azure via metadata server
        """
        self.logger.debug('Checking Azure metadata')
        try:
            response = requests.get(self.metadata_url, headers=self.headers,)
            return response.status_code == 200
        except BaseException:
            return False

    def check_vendor_file(self):
        """
            Tries to identify Azure provider by reading the /sys/class/dmi/id/sys_vendor
        """
        self.logger.debug('Checking Azure vendor file')
        do_path = Path(self.vendor_file)
        if do_path.is_file():
            if 'Microsoft Corporation' in open(self.vendor_file).read():
                return True
        return False



from abc import ABCMeta  # noqa: F401
from abc import abstractmethod  # noqa: F401


class AbstractProvider:
    """
        Abstract class representing a cloud provider.
        All concrete cloud providers should implement this.
    """

    @abstractmethod
    def identify(self):
        pass  # pragma: no cover

    @abstractmethod
    def check_metadata_server(self):
        pass  # pragma: no cover

    @abstractmethod
    def check_vendor_file(self):
        pass  # pragma: no cover
